#include "m_pd.h"
#include <cuda_runtime_api.h>
#include "pdcuda_api.h"

t_signal *cuda_signal_freelist[MAXLOGSIG+1];
t_signal *cuda_signal_freeborrowed;
t_signal *cuda_signal_usedlist;

t_int *cuda_zero_perform(t_int *w);
t_int *cuda_sig_tilde_perform(t_int *w);
t_int *cuda_plus_perform(t_int *w);

extern int ugen_loud;

t_signal *cuda_signal_new(int n, t_float sr)
{
    int logn, n2, vecsize = 0;
    t_signal *ret, **whichlist;
    t_sample *fp;
    logn = ilog2(n);
    if (n)
    {
        if ((vecsize = (1<<logn)) != n)
            vecsize *= 2;
        if (logn > MAXLOGSIG)
            bug("signal buffer too large");
        whichlist = cuda_signal_freelist + logn;
    }
    else
        whichlist = &cuda_signal_freeborrowed;

        /* first try to reclaim one from the free list */
    if (ret = *whichlist)
        *whichlist = ret->s_nextfree;
    else
    {
            /* LATER figure out what to do for out-of-space here! */
        ret = (t_signal *)t_getbytes(sizeof *ret);
        if (n)
        {
            //ret->s_vec = (t_sample *)getbytes(vecsize * sizeof (*ret->s_vec));
            cudaMalloc((void **)&ret->s_vec, vecsize * sizeof(*ret->s_vec));
            ret->s_isborrowed = 0;
        }
        else
        {
            ret->s_vec = 0;
            ret->s_isborrowed = 1;
        }
        ret->s_nextused = cuda_signal_usedlist;
        cuda_signal_usedlist = ret;
    }
    ret->s_n = n;
    ret->s_vecsize = vecsize;
    ret->s_sr = sr;
    ret->s_refcount = 0;
    ret->s_borrowedfrom = 0;
    if (ugen_loud) post("cuda_new: %lx (%lx) %d", ret, ret->s_vec, ret->s_isborrowed);
    return (ret);
}

t_signal *cuda_signal_newlike(const t_signal *sig)
{
    return (cuda_signal_new(sig->s_n, sig->s_sr));
}

void cuda_signal_cleanup(void)
{
    t_signal **svec, *sig, *sig2;
    int i;
    while (sig = cuda_signal_usedlist)
    {
        cuda_signal_usedlist = sig->s_nextused;
        if (!sig->s_isborrowed)
            cudaFree(sig->s_vec);
        t_freebytes(sig, sizeof *sig);
    }
    for (i = 0; i <= MAXLOGSIG; i++)
        cuda_signal_freelist[i] = 0;
    cuda_signal_freeborrowed = 0;
}

void cuda_signal_makereusable(t_signal *sig)
{
    int logn = ilog2(sig->s_vecsize);
#if 1
    t_signal *s5;
    for (s5 = cuda_signal_freeborrowed; s5; s5 = s5->s_nextfree)
    {
        if (s5 == sig)
        {
            bug("signal_free 3");
            return;
        }
    }
    for (s5 = cuda_signal_freelist[logn]; s5; s5 = s5->s_nextfree)
    {
        if (s5 == sig)
        {
            bug("signal_free 4");
            return;
        }
    }
#endif
    if (ugen_loud) post("cuda_free %lx (%lx): %d", sig, sig->s_vec, sig->s_isborrowed);
    if (sig->s_isborrowed)
    {
            /* if the signal is borrowed, decrement the borrowed-from signal's
                reference count, possibly marking it reusable too */
        t_signal *s2 = sig->s_borrowedfrom;
        if ((s2 == sig) || !s2)
            bug("signal_free");
        s2->s_refcount--;
        if (!s2->s_refcount)
            cuda_signal_makereusable(s2);
        sig->s_nextfree = cuda_signal_freeborrowed;
        cuda_signal_freeborrowed = sig;
    }
    else
    {
            /* if it's a real signal (not borrowed), put it on the free list
                so we can reuse it. */
        if (cuda_signal_freelist[logn] == sig) bug("signal_free 2");
        sig->s_nextfree = cuda_signal_freelist[logn];
        cuda_signal_freelist[logn] = sig;
    }
}

t_int *cuda_zero_perform(t_int *w)   /* zero out a vector */
{
    t_sample *out = (t_sample *)(w[1]);
    t_int n = (int)(w[2]);
    cuda_zero_basic(out, n);
    return (w+3);
}

void cuda_dsp_add_zero(t_sample *out, int n)
{
    dsp_add(cuda_zero_perform, 2, out, n);
}

t_int *cuda_sig_tilde_perform(t_int *w)
{
    t_float f = *(t_float *)(w[1]);
    t_sample *out = (t_sample *)(w[2]);
    int n = (int)(w[3]);
    cuda_sig_basic(out, f, n);
    return (w+4);
}

void cuda_dsp_add_scalarcopy(t_float *in, t_sample *out, int n)
{
    dsp_add(cuda_sig_tilde_perform, 3, in, out, n);
}

t_int *cuda_scalarplus_perform(t_int *w)
{
    t_sample *in = (t_sample *)(w[1]);
    t_float f = *(t_float *)(w[2]);
    t_sample *out = (t_sample *)(w[3]);
    int n = (int)(w[4]);
    cuda_scalarplus_basic(in, out, f, n);
    return (w+5);
}

void cuda_dsp_add_plus(t_sample *in1, t_sample *in2, t_sample *out, int n)
{
    dsp_add(cuda_plus_perform, 4, in1, in2, out, n);
}

t_int *cuda_plus_perform(t_int *w)
{
    t_sample *in1 = (t_sample *)(w[1]);
    t_sample *in2 = (t_sample *)(w[2]);
    t_sample *out = (t_sample *)(w[3]);
    int n = (int)(w[4]);
    cuda_plus_basic(in1, in2, out, n);
    return (w+5);
}

typedef struct _vinlet
{
    t_object x_obj;
    t_canvas *x_canvas;
    t_inlet *x_inlet;
    int x_bufsize;
    t_float *x_buf;         /* signal buffer; zero if not a signal */
    t_float *x_endbuf;
    t_float *x_fill;
    t_float *x_read;
    int x_hop;
  /* if not reblocking, the next slot communicates the parent's inlet
     signal from the prolog to the DSP routine: */
    t_signal *x_directsignal;

  t_resample x_updown;
} t_vinlet;

t_int *cuda_vinlet_doprolog(t_int *w)
{
    t_vinlet *x = (t_vinlet *)(w[1]);
    t_float *in = (t_float *)(w[2]);
    int n = (int)(w[3]);
    enum cudaMemcpyKind kind = (enum cudaMemcpyKind)(w[4]);
    t_float *out = x->x_fill;
    if (out == x->x_endbuf)
    {
      t_float *f1 = x->x_buf, *f2 = x->x_buf + x->x_hop;
        int nshift = x->x_bufsize - x->x_hop;
        out -= x->x_hop;
        //while (nshift--) *f1++ = *f2++;
        cudaMemcpy((void *) f1, (void *) f2, nshift*sizeof(*f2), kind);
    }
#if 0
    if (tot < 5) post("in %lx out %lx n %lx", in, out, n), tot++;
    if (tot < 5) post("in[0] %f in[1] %f in[2] %f", in[0], in[1], in[2]);
#endif

    cudaMemcpy((void *) out, (void *) in, n*sizeof(*in), kind);
    x->x_fill = out + n;
    return (w+5);
}

t_int *cuda_vinlet_perform(t_int *w)
{
    t_vinlet *x = (t_vinlet *)(w[1]);
    t_float *out = (t_float *)(w[2]);
    int n = (int)(w[3]);
    t_float *in = x->x_read;
#if 0
    if (tot < 5) post("-in %lx out %lx n %d", in, out, n);
    if (tot < 5) post("-buf %lx endbuf %lx", x->x_buf, x->x_endbuf);
    if (tot < 5) post("in[0] %f in[1] %f in[2] %f", in[0], in[1], in[2]);
#endif
    cudaMemcpy((void *) out, (void *) in, n*sizeof(*in), cudaMemcpyDeviceToDevice);
    in+=n;
    if (in == x->x_endbuf) in = x->x_buf;
    x->x_read = in;
    return (w+4);
}

t_int *cuda_copy_perform(t_int *w)
{
    t_sample *in = (t_sample *)(w[1]);
    t_sample *out = (t_sample *)(w[2]);
    int n = (int)(w[3]);
    enum cudaMemcpyKind kind = (enum cudaMemcpyKind)(w[4]);    
    cudaMemcpy((void *) out, (void *) in, n*sizeof(*in), kind);
    return (w+5);
}

typedef struct _voutlet
{
    t_object x_obj;
    t_canvas *x_canvas;
    t_outlet *x_parentoutlet;
    int x_bufsize;
    t_sample *x_buf;         /* signal buffer; zero if not a signal */
    t_sample *x_endbuf;
    t_sample *x_empty;       /* next to read out of buffer in epilog code */
    t_sample *x_write;       /* next to write in to buffer */
    int x_hop;              /* hopsize */
        /* vice versa from the inlet, if we don't block, this holds the
        parent's outlet signal, valid between the prolog and the dsp setup
        routines.  */
    t_signal *x_directsignal;
        /* and here's a flag indicating that we aren't blocked but have to
        do a copy (because we're switched). */
    char x_justcopyout;
  t_resample x_updown;
} t_voutlet;

t_int *cuda_voutlet_perform(t_int *w)
{
    t_voutlet *x = (t_voutlet *)(w[1]);
    t_float *in = (t_float *)(w[2]);
    int n = (int)(w[3]);
    t_sample *out = x->x_write, *outwas = out;
#if 0
    if (tot < 5) post("-in %lx out %lx n %d", in, out, n);
    if (tot < 5) post("-buf %lx endbuf %lx", x->x_buf, x->x_endbuf);
#endif
    if (out+n > x->x_endbuf)
    {
        size_t sz1=x->x_endbuf-out;
        size_t sz2=n*sizeof(*in)-sz1;
        cudaMemcpy((void *) out, (void *) in, sz1,
            cudaMemcpyDeviceToDevice);
        cudaMemcpy((void *) x->x_buf, (void *) in+sz1, sz2,
            cudaMemcpyDeviceToDevice);    
        out = x->x_buf + sz2;
    }
    outwas += x->x_hop;
    if (outwas >= x->x_endbuf) outwas = x->x_buf;
    x->x_write = outwas;
    return (w+4);
}

t_int *cuda_voutlet_doepilog(t_int *w)
{
    t_voutlet *x = (t_voutlet *)(w[1]);
    t_sample *out = (t_sample *)(w[2]);

    int n = (int)(w[3]);
    enum cudaMemcpyKind kind = (enum cudaMemcpyKind)(w[4]);
    t_sample *in = x->x_empty;
#if 0
    if (tot < 5) post("outlet in %lx out %lx n %lx", in, out, n), tot++;
#endif 
    //for (; n--; in++) *out++ = *in, *in = 0;
    cudaMemcpy((void *) out, (void *) in, n*sizeof(*in), kind);
    cuda_zero_basic(in, n);
    if (in == x->x_endbuf) in = x->x_buf;
    x->x_empty = in;
    return (w+5);
}


