#ifndef CUPV_H
#define CUPV_H

#include <driver_types.h>
#include <cufft.h>
#include <cuda_runtime_api.h>

#include "cutil.h"
#include "m_pd.h"
#include "pdcuda_api.h"



typedef struct _cupv
{
  t_object x_obj;       /* obligatory header */
  t_float x_f;          /* place to hold inlet's value if it's set by message */

  /* DSP variables */
  int veclength;
  int cycles;
  t_float alpha;
  t_float beta;
  t_float deltat;
  int overlap;
  t_float srate;

  /* time measurement */
  t_float timeHostToDevice; 
  t_float timeKernel; 
  t_float timeDeviceToHost; 
  t_outlet *f_timeHostToDevice; 
  t_outlet *f_timeKernel; 
  t_outlet *f_timeDeviceToHost; 
  cudaEvent_t start, stop;

  /* pointers for memory on host */
  /*cufftReal *h_A;*/ 

  /* pointers for memory on device */
  cufftReal *d_A;
  cufftComplex *d_B;
  t_float *d_C;
  t_float *d_D;
  t_float *d_E;
  t_float *d_F;
  t_float *d_G;
  t_float *d_H;
  t_float *tmp;

  /* CUFFT */
  cufftHandle plan;

  /* sine table */
  t_float *h_sine; /* on host */
  t_float *d_sine; /* on device */
  int s;
  
  /* CUDA texture */
  struct textureReference *sineTextureRef;
  struct cudaChannelFormatDesc sineTextureFormatDesc;
  size_t sineTextureOffset;
  struct cudaArray* sineTextureArray;

} t_cupv;

#endif
