#!/bin/sh

for file in ../roundtrip/*.txt; do
  if [ -z "`echo ${file} | grep \"block-size\"`" ]; then
    name=`basename ${file} .txt`
    title=`echo ${name} | tr '-' ' '`
    if [ ! -z "`echo ${file} | grep -v block-period`" ]; then
      if [ ! -z "`echo ${file} | grep FFT`" ]; then 
        if [ ! -z "`echo ${file} | grep 275`" ]; then
          model="GTX275"
          key='off'
          height="0.5"
          width="0.5"
          rmargin=3
        elif [ ! -z "`echo ${file} | grep 470`" ]; then
          model="GTX470"
          key='off'
          height="0.5"
          width="0.5"
          rmargin=3
        elif [ ! -z "`echo ${file} | grep 4000`" ]; then
          model="GF100GL"
          key='above vertical'
          height="0.665"
          width="0.5"
          rmargin=3
        fi
        ./roundtrip-fft.sh ${file} ../img/${name}.eps "${model}" "${key}"  ${height} ${width} ${rmargin}
        ./roundtrip-fft.2.sh ${file} ../img/${name}-2.eps "${model}" "${key}"  ${height} ${width} ${rmargin}
        epspdf ../img/${name}-2.eps ../img/${name}-2.pdf
        epspdf ../img/${name}.eps ../img/${name}.pdf
      fi
      if [ ! -z "`echo ${file} | grep CONV`" ]; then 
        if [ ! -z "`echo ${file} | grep 275`" ]; then
          model="GTX275"
          key='off'
          height="0.5"
          width="0.5"
          rmargin=3
        elif [ ! -z "`echo ${file} | grep 470`" ]; then
          model="GTX470"
          key='off'
          height="0.5"
          width="0.5"
          rmargin=3
        elif [ ! -z "`echo ${file} | grep 4000`" ]; then
          model="GF100GL"
          key='above vertical'
          height="0.665"
          width="0.5"
          rmargin=3
        fi
        ./roundtrip-conv.sh ${file} ../img/${name}.eps "${model}"
        ./roundtrip-conv.2.sh ${file} ../img/${name}-2.eps "${model}" "${key}"  ${height} ${width} ${rmargin}
        epspdf ../img/${name}.eps ../img/${name}.pdf
        epspdf ../img/${name}-2.eps ../img/${name}-2.pdf
      else
        #./roundtrip-pv.sh ${file} ../img/${name}.eps "${title}"
        #epspdf ../img/${name}.eps ../img/${name}.pdf
        echo oi
      fi
    fi
  fi
done

./roundtrip-conv.all.sh
epspdf ../img/CONV-all.eps ../img/CONV-all.pdf
epspdf ../img/CONV-all-zoom.eps ../img/CONV-all-zoom.pdf

for model in 275 470 4000; do
  DATAFILE1=../roundtrip/PV-${model}-nointerp-batch.txt
  DATAFILE2=../roundtrip/PV-${model}-linear-batch.txt
  DATAFILE3=../roundtrip/PV-${model}-cubic-batch.txt
  DATAFILE4=../roundtrip/PV-${model}-builtin-batch.txt
  DATAFILE5=../roundtrip/PV-${model}-nocalc-batch.txt
  DATAFILE6=../roundtrip/PV-${model}-texture-batch.txt
  if [ "${model}" -eq "275" ]; then
    mod="GTX275"
    key="off"
    height="0.5"
    width="0.5"
  elif [ "${model}" -eq "470" ]; then
    mod="GTX470"
    key="off"
    height="0.5"
    width="0.5"
  elif [ "${model}" -eq "4000" ]; then
    mod="GF100GL"
    key="above horizontal"
    height="0.7"
    width="0.50"
  fi
  ./roundtrip-pv-interp.sh ${DATAFILE1} ${DATAFILE2} ${DATAFILE3} \
    ${DATAFILE4} ${DATAFILE5} ${DATAFILE6} ../img/PV-${model}-1-interp.eps \
    ${mod} "${key}" ${height}
  epspdf ../img/PV-${model}-1-interp.eps ../img/PV-${model}-1-interp.pdf
  ./roundtrip-pv-interp.2.sh ${DATAFILE1} ${DATAFILE2} ${DATAFILE3} \
    ${DATAFILE4} ${DATAFILE5} ${DATAFILE6} ../img/PV-${model}-2-interp.eps \
    ${mod} "${key}" ${height} 
  epspdf ../img/PV-${model}-2-interp.eps ../img/PV-${model}-2-interp.pdf

  # memory transfer times comparison
  DATAFILE1=../roundtrip/FFT-${model}-batch.txt
  DATAFILE2=../roundtrip/CONV-${model}-batch.txt
  DATAFILE3=../roundtrip/PV-${model}-cubic-batch.txt
  ./memory-comparison.sh ${DATAFILE1} ${DATAFILE2} ${DATAFILE3} \
    ../img/MEM-${model}.eps "Memory transfer times - ${mod}" "${key}" \
    ${height} ${width}
  epspdf ../img/MEM-${model}.eps ../img/MEM-${model}.pdf
done

rm -rf ../img/*.eps
