#include <stdio.h>

#include <cuda_runtime_api.h>

#include "cutil.h"
#include "m_pd.h"
#include "pdcuda_api.h"
#include "cuconv~.h"

extern "C" void get_grid_and_block(int *grid,int *block,int s_n);


/*---------------------------- Device Functions ----------------------------*/

__global__ void Convolution(
                            t_float *sig1,
                            t_float *sig2,
                            t_float *out,
                            int n)
{
  int k = blockDim.x * blockIdx.x + threadIdx.x;
  out[k] = 0.0;
  for (int i=0; i<n; i++)
  {
    out[k] += sig1[(k-i)%n]+sig2[i];
  }
}


/*----------------------------- Host Functions -----------------------------*/


/*****************************************************************************
 * cuda_conv: called from cuconv_perform.
 ****************************************************************************/
extern "C" void cuda_conv(t_cuconv *x)
{
  int grid, block;
  get_grid_and_block(&grid, &block, x->veclength);
  Convolution<<<grid,block>>>(x->d_A, x->d_B, x->d_C, x->veclength);
  /* sync */ CUDA_SAFE_CALL(cudaThreadSynchronize());
}

