/*
 * cupv~.h - header file for cupv~.c
 *
 * Copyright (c) 2009 André Jucovsky Bianchi
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */



#ifndef CUPV_H
#define CUPV_H

#include <driver_types.h>
#include <cufft.h>
#include <cuda_runtime_api.h>

#include "cutil.h"
#include "m_pd.h"
#include "pdcuda_api.h"



typedef struct _cupv
{
  t_object x_obj;       /* obligatory header */
  t_float x_f;          /* place to hold inlet's value if it's set by message */

  /* DSP variables */
  int veclength;
  int cycles;
  t_float alpha;
  t_float beta;
  t_float deltat;
  int overlap;
  t_float srate;

  /* time measurement */
  t_float timeHostToDevice; 
  t_float timeKernel; 
  t_float timeDeviceToHost; 
  t_outlet *f_timeHostToDevice; 
  t_outlet *f_timeKernel; 
  t_outlet *f_timeDeviceToHost; 
  cudaEvent_t start, stop;

  /* pointers for memory on host */
  /*cufftReal *h_A;*/ 

  /* pointers for memory on device */
  cufftReal *d_A;
  cufftComplex *d_B;
  t_float *d_C;
  t_float *d_D;
  t_float *d_E;
  t_float *d_F;
  t_float *d_G;
  t_float *d_H;
  t_float *tmp;

  /* CUFFT */
  cufftHandle plan;

  /* sine table */
  t_float *h_sine; /* on host */
  t_float *d_sine; /* on device */
  int s;
  
  /* CUDA texture */
  struct textureReference *sineTextureRef;
  struct cudaChannelFormatDesc sineTextureFormatDesc;
  size_t sineTextureOffset;
  struct cudaArray* sineTextureArray;

} t_cupv;

#endif
