/*
 * This software contains source code provided by NVIDIA Corporation.
 */

#ifndef _CUTIL_H_
#define _CUTIL_H_

#include <stdio.h>
#include <stdlib.h>

#ifdef __cplusplus
extern "C" {
#endif

#  define CUDA_SAFE_CALL_NO_SYNC(call) {                                     \
    cudaError_t err = call;                                                  \
    if( cudaSuccess != err) {                                                \
        fprintf(stderr, "Cuda error in file '%s' in line %i : %s.\n",        \
                __FILE__, __LINE__, cudaGetErrorString( err) );              \
        exit(EXIT_FAILURE);                                                  \
    } }


#  define CUDA_SAFE_CALL(call)     CUDA_SAFE_CALL_NO_SYNC(call);             \


#  define CUFFT_SAFE_CALL( call) {                                           \
    cufftResult err = call;                                                  \
    if( CUFFT_SUCCESS != err) {                                              \
        fprintf(stderr, "CUFFT error in file '%s' in line %i.\n",            \
                __FILE__, __LINE__);                                         \
        exit(EXIT_FAILURE);                                                  \
    } }


#  define CUDA_SAFE_MEASURE(call, start, stop, ptime, stream) {              \
    cudaEventRecord(start, stream);                                          \
    cudaError_t err = call;                                                  \
    cudaEventRecord(stop, stream);                                           \
    if (cudaSuccess != err) {                                                \
      fprintf(stderr, "Cuda error in file '%s' in line %i : %s.\n",          \
              __FILE__, __LINE__, cudaGetErrorString(err));                  \
      exit(EXIT_FAILURE);                                                    \
    }                                                                        \
    cudaEventSynchronize(stop);                                              \
    cudaEventElapsedTime(ptime, start, stop);                                \
  }

#  define CUT_SAFE_MALLOC( mallocCall ) {                                    \
    cudaError_t err = mallocCall;                                            \
    if(err != cudaSuccess) {                                                 \
        fprintf(stderr, "Malloc failure in file '%s' in line %i : %s.\n",    \
                __FILE__, __LINE__, cudaGetErrorString(err));                \
        exit(EXIT_FAILURE);                                                  \
    } }


#ifdef __cplusplus
}
#endif

#endif  // #ifndef _CUTIL_H_
