#include <stdio.h>

#include <cuda_runtime_api.h>

#include "m_pd.h"

extern "C" void get_grid_and_block(int *grid,int *block,int s_n);

__global__ void Zero(float* A)
{
  int i = blockDim.x * blockIdx.x + threadIdx.x;
  A[i] = 0;
}

__global__ void Sig(float* A, float f)
{
    int i = blockDim.x * blockIdx.x + threadIdx.x;
    A[i] = f;
}

__global__ void Plus(float* A, float* B, float *C)
{
    int i = blockDim.x * blockIdx.x + threadIdx.x;
    C[i] = A[i] + B[i];
}

__global__ void ScalarPlus(float* A, float* B, float f)
{
    int i = blockDim.x * blockIdx.x + threadIdx.x;
    B[i] = A[i] + f;
}

extern "C" void cuda_zero_basic(t_sample *vec, int n)
{
    int grid,block;
    cudaError_t errno;
    get_grid_and_block(&grid, &block, n);
    Zero<<<grid,block>>>(vec);
    /*cudaThreadSynchronize();*/
    errno = cudaGetLastError();
    if (errno)
      printf("cuda error (cuda_zero_basic): %d - %s\n", (int) errno, cudaGetErrorString(errno));

}

extern "C" void cuda_sig_basic(t_sample *vec, t_float f, int n)
{
    int grid, block;
    get_grid_and_block(&grid, &block, n);
    Sig<<<grid,block>>>(vec, f);
}

extern "C" void cuda_scalarplus_basic(t_sample *in, t_sample *out, t_float f, int n)
{
    int grid, block;
    get_grid_and_block(&grid,&block,n);
    cudaThreadSynchronize();
    ScalarPlus<<<grid,block>>>(in, out, f);
}

extern "C" void cuda_plus_basic(t_sample *in1, t_sample *in2, t_sample *out, int n)
{
    int grid, block;
    get_grid_and_block(&grid,&block,n);
    Plus<<<grid,block>>>(in1, in2, out);
}

