/* ajb@ime.usp.br */

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <cuda_runtime_api.h>

#include "m_pd.h"
#include "pdcuda_api.h"




/* --------------------------- device functions --------------------------- */


float *d_A, *d_B;

void get_grid_and_block(int *grid,int *block,int s_n);


void post(char *s);
/* ------------------------ cuda~ ----------------------------- */

static t_class *cuda_class;

typedef struct _cuda
{
  t_object x_obj;       /* obligatory header */
  t_float x_f;          /* place to hold inlet's value if it's set by message */
  t_outlet *bangoutlet; /* saida dizendo que acabou o processamento */
} t_cuda;


static void cuda_bang(t_cuda *x) {
  post("bang!");
}

static void cuda_anything(t_cuda *x, t_symbol *s, int argc, t_atom *argv)
{
}

/* this is the actual performance routine which acts on the samples.
   It's called with a single pointer "w" which is our location in the
   DSP call list.  We return a new "w" which will point to the next item
   after us.  Meanwhile, w[0] is just a pointer to dsp-perform itself
   (no use to us), w[1] and w[2] are the input and output vector locations,
   and w[3] is the number of points to calculate. */
static t_int *cuda_perform(t_int *w)
{
  t_float *in = (t_float *)(w[1]);
  t_float *out1 = (t_float *)(w[2]);
  t_float *out2 = (t_float *)(w[3]);
  int n = (int)(w[4]);
  t_cuda *x = (t_cuda *)(w[5]);

  cudaError_t errno;


  errno = cudaMemcpy(d_A, in, (size_t) n*sizeof(t_sample), cudaMemcpyHostToDevice);
  if (errno)
    printf("cuda error (perform): %d - %s\n", (int) errno, cudaGetErrorString(errno));

  /*cuda_scalarplus_basic(d_A, d_B, 3.0, n);*/

  errno = cudaMemcpy(d_B, d_A, (size_t) n*sizeof(t_sample), cudaMemcpyDeviceToDevice);

  cudaSetDeviceFlags(cudaDeviceBlockingSync);
  cuda_zero_basic(d_B, n);
  cudaDeviceSynchronize();

  errno = cudaMemcpy(out2, d_B, (size_t) n*sizeof(t_sample), cudaMemcpyDeviceToHost);
  if (errno)
    printf("cuda error (perform): %d - %s\n", (int) errno, cudaGetErrorString(errno));

  /*while (n--) {
    out2[n] = in[n] + 2.0;
    out1[n] = in[n] + 1.0;
  }*/

  return (w+6);
}

/* called to start DSP.  Here we call Pd back to add our perform
   routine to a linear callback list which Pd in turn calls to grind
   out the samples. */
static void cuda_dsp(t_cuda *x, t_signal **sp)
{
  dsp_add(cuda_perform, 5, sp[0]->s_vec, sp[1]->s_vec, sp[2]->s_vec, sp[0]->s_n, x);
}


/**
 * cuda_new(void)
 */
static void *cuda_new(void)
{
  int N = 64;
  cudaError_t errno;
  
  post("initializing cuda~ object.");

  /* init pd object */
  t_cuda *x = (t_cuda *)pd_new(cuda_class);
  outlet_new(&x->x_obj, gensym("signal"));
  outlet_new(&x->x_obj, gensym("signal"));

  lib_set_cuda_device(1);

  /* device memory */
  errno = cudaMalloc((void **) &d_A, N*sizeof(t_sample));
  if (errno)
    printf("cuda error (new): %d - %s\n", (int) errno, cudaGetErrorString(errno));

  errno = cudaMalloc((void **) &d_B, N*sizeof(t_sample));
  if (errno)
    printf("cuda error (new): %d - %s\n", (int) errno, cudaGetErrorString(errno));


  return (x);
}


/**
 * cuda_new(void)
 *
 * this routine, which must have exactly this name (with the "~" replaced
 * by "_tilde) is called when the code is first loaded, and tells Pd how
 * to build the "class".
 */
void cuda_tilde_setup(void)
{
  post("inicializando classe cuda~...");
  cuda_class = class_new(gensym("cuda~"), (t_newmethod)cuda_new, 0,
                                  sizeof(t_cuda), 0, A_DEFFLOAT, 0);
  /* this is magic to declare that the leftmost, "main" inlet
     takes signals; other signal inlets are done differently...*/
  CLASS_MAINSIGNALIN(cuda_class, t_cuda, x_f);
  /* here we tell Pd about the "dsp" method, which is called back
     when DSP is turned on. */
  class_addmethod(cuda_class, (t_method)cuda_dsp, gensym("dsp"), (t_atomtype) 0);
  class_addbang(cuda_class, (t_method)cuda_bang);
  class_addanything(cuda_class, cuda_anything);
}
