/* ajb@ime.usp.br */

#include <cuda_runtime_api.h>
#include "m_pd.h"
#include <stdio.h>

extern "C" void get_grid_and_block(int *grid,int *block,int s_n, int threads_per_block, int max_pd_block_size)
{
    if (s_n > threads_per_block)
    {
        *grid = s_n/threads_per_block;
        *block = threads_per_block;
    }
    else
    {
        *grid = 1;
        *block = s_n;
    }
    /*printf("grid: %d, block: %d\n", *grid, *block);*/
}


//place-holder for array work to do

// CUDA device registration code goes here:
// returns 0 if successful or not necessary, 1 for error

__global__ void Zero(float* A)
{
  int i = blockDim.x * blockIdx.x + threadIdx.x;
  A[i] = 0;
}

__global__ void Sig(float* A, float f)
{
    int i = blockDim.x * blockIdx.x + threadIdx.x;
    A[i] = f;
}

__global__ void Plus(float* A, float* B, float *C)
{
    int i = blockDim.x * blockIdx.x + threadIdx.x;
    C[i] = A[i] + B[i];
}

__global__ void ScalarPlus(float* A, float* B, float f)
{
    int i = blockDim.x * blockIdx.x + threadIdx.x;
    B[i] = A[i] + f;
}

extern "C" void cuda_zero_basic(t_sample *vec, int n,  int threads_per_block, int max_pd_block_size)
{
    int grid,block;
    cudaError_t errno;
    get_grid_and_block(&grid, &block, n, threads_per_block, max_pd_block_size);
    Zero<<<grid,block>>>(vec);
    cudaThreadSynchronize();
    errno = cudaGetLastError();
    if (errno)
      printf("cuda error (cuda_zero_basic): %d - %s\n", (int) errno, cudaGetErrorString(errno));

}

extern "C" void cuda_sig_basic(t_sample *vec, t_float f, int n,  int threads_per_block, int max_pd_block_size)
{
    int grid, block;
    get_grid_and_block(&grid, &block, n, threads_per_block, max_pd_block_size);
    Sig<<<grid,block>>>(vec, f);
}

extern "C" void cuda_scalarplus_basic(t_sample *in, t_sample *out, t_float f, int n,  int threads_per_block, int max_pd_block_size)
{
    int grid, block;
    get_grid_and_block(&grid,&block,n, threads_per_block, max_pd_block_size);
    ScalarPlus<<<grid,block>>>(in, out, f);
}

extern "C" void cuda_plus_basic(t_sample *in1, t_sample *in2, t_sample *out, int n,  int threads_per_block, int max_pd_block_size)
{
    int grid, block;
    get_grid_and_block(&grid,&block,n, threads_per_block, max_pd_block_size);
    Plus<<<grid,block>>>(in1, in2, out);
}

