// pdcuda_cc.c
// contains CC-compilable code
// publicly available functions of pdcuda 
#include<cuda_runtime_api.h>
#include"m_pd.h"
#include <stdio.h>

// Used for default scheduling, set these values at setup time by choosing the device
int threads_per_block;  // Set as 128 or 256, typically. use 512 only if > 256 cores
int max_pd_block_size; // Set as threads_per_block * current_cudaDeviceProp.maxGridSize[0]

void get_grid_and_block(int *grid,int *block,int s_n)
{
    if (s_n > threads_per_block)
    {
        *grid = s_n/threads_per_block;
        *block = threads_per_block;
    }
    else
    {
        *grid = 1;
        *block = s_n;
    }
    /*printf("grid: %d, block: %d\n", *grid, *block);*/
}

//Note: the compiler switch is -DWARP16 to compile with 16-wide warps
#ifdef WARP16
#define WARP_SIZE 16
#else
#define WARP32
#define WARP_SIZE 32
#endif

//Boolean variable for cuda stream state--use as check against failed startup
int cuda_stream = 0;

// The stream itself--can be made array of streams
cudaStream_t glob_cuda_stream;

// counts canvases--check for cleanup
int cuda_canvas_num = 0; 

//Once set, the following variables are to remain the same until the process ends
struct cudaDeviceProp current_cudaDeviceProp;

int cuda_current_device_num = 0;
int cuda_devicecount = 0;

//place-holder for array work to do
static int cuda_tabcount = 0;

// CUDA device registration code goes here:
// returns 0 if successful or not necessary, 1 for error
int lib_set_cuda_device(int device)
{
    if (!cuda_stream)
    {
        if (device >= 0)
        {
            printf("Detecting CUDA devices");
            cudaGetDeviceCount(&cuda_devicecount);
            if (cuda_devicecount)
            {
                cudaError_t error_device = cudaSetDevice(device); 
                cudaGetDeviceProperties(&current_cudaDeviceProp, device);
                printf("Loading CUDA device %d", device);
                printf(cudaGetErrorString(error_device));
                // If there's an error, fall back on device 0,
                cudaError_t error_stream;
                if (!error_device)
                {
                    /*error_stream = cudaStreamCreate(&glob_cuda_stream);
                    if(!error_stream)
                        cuda_stream = 1;
                    else
                    {
                        printf("Error Creating CUDA Stream");
                        printf(cudaGetErrorString(error_stream));
                    }*/
                    cuda_current_device_num = device;
                    printf("Device %d registered", device);
                    threads_per_block = 256;
                    max_pd_block_size = threads_per_block * current_cudaDeviceProp.maxGridSize[0];
                    printf("CUDA Block size: %d",threads_per_block);
                    printf("Max Pd Block size: %d",max_pd_block_size);
                }
                else
                {
                    printf("Failed to load CUDA device %d", device);
                    printf("Falling back on default device 0");
                    device = 0;
                    error_device = cudaSetDevice(device);
                    cudaGetDeviceProperties(&current_cudaDeviceProp, device);
                    printf("Loading CUDA device %d", device);
                    printf(cudaGetErrorString(error_device));
                    if (!error_device)
                    {
                        error_stream = cudaStreamCreate(&glob_cuda_stream);
                        if(!error_stream)
                            cuda_stream = 1;
                        else
                        {
                            printf("Error Creating CUDA Stream");
                            printf(cudaGetErrorString(error_stream));
                        }
                        cuda_current_device_num = device;
                        printf("Device %d registered", device);
                        threads_per_block = 256;
                        max_pd_block_size = threads_per_block * current_cudaDeviceProp.maxGridSize[0];
                        printf("CUDA Block size: %d",threads_per_block);
                        printf("Max Pd Block size: %d",max_pd_block_size);
                    }
                }
            }
            else
                printf("No CUDA devices detected");
        }
        else
            printf("Device number %d not recognized", device);
    }
    /*if (!cuda_stream)
    {
        printf("No CUDA device registered");
        return(1);
    }*/
    return(0);
}

