#include <stdio.h>
#include <dlfcn.h>
#include "m_pd.h"
#include <cuda_runtime_api.h>

void cuda_zero_basic(t_sample *vec, int n, int threads_per_block, int max_pd_block_size);

int threads_per_block;  // Set as 128 or 256, typically. use 512 only if > 256 cores
int max_pd_block_size; // Set as threads_per_block * current_cudaDeviceProp.maxGridSize[0]

//Note: the compiler switch is -DWARP16 to compile with 16-wide warps
#ifdef WARP16
#define WARP_SIZE 16
#else
#define WARP32
#define WARP_SIZE 32
#endif

//Boolean variable for cuda stream state--use as check against failed startup
int cuda_stream = 0;

// The stream itself--can be made array of streams
cudaStream_t glob_cuda_stream;

// counts canvases--check for cleanup
int cuda_canvas_num = 0; 

//Once set, the following variables are to remain the same until the process ends
struct cudaDeviceProp current_cudaDeviceProp;

int cuda_current_device_num = 0;
int cuda_devicecount = 0;

int lib_set_cuda_device(int device)
{
    if (!cuda_stream)
    {
        if (device >= 0)
        {
            printf("Detecting CUDA devices\n");
int threads_per_block;  // Set as 128 or 256, typically. use 512 only if > 256 cores
int max_pd_block_size; // Set as threads_per_block * current_cudaDeviceProp.maxGridSize[0]
            cudaGetDeviceCount(&cuda_devicecount);
            if (cuda_devicecount)
            {
                cudaError_t error_device = cudaSetDevice(device); 
                cudaGetDeviceProperties(&current_cudaDeviceProp, device);
                printf("Loading CUDA device %d\n", device);
                printf("%s\n", cudaGetErrorString(error_device));
                // If there's an error, fall back on device 0,
                cudaError_t error_stream;
                if (!error_device)
                {
                    /*error_stream = cudaStreamCreate(&glob_cuda_stream);
                    if(!error_stream)
                        cuda_stream = 1;
                    else
                    {
                        printf("Error Creating CUDA Stream");
                        printf(cudaGetErrorString(error_stream));
                    }*/
                    cuda_current_device_num = device;
                    printf("Device %d registered\n", device);
                    threads_per_block = 256;
                    max_pd_block_size = threads_per_block * current_cudaDeviceProp.maxGridSize[0];
                    printf("CUDA Block size: %d\n",threads_per_block);
                    printf("Max Pd Block size: %d\n",max_pd_block_size);
                }
                else
                {
                    printf("Failed to load CUDA device %d\n", device);
                    printf("Falling back on default device 0\n");
                    device = 0;
                    error_device = cudaSetDevice(device);
                    cudaGetDeviceProperties(&current_cudaDeviceProp, device);
                    printf("Loading CUDA device %d\n", device);
                    printf("%s\n", cudaGetErrorString(error_device));
                    if (!error_device)
                    {
                        error_stream = cudaStreamCreate(&glob_cuda_stream);
                        if(!error_stream)
                            cuda_stream = 1;
                        else
                        {
                            printf("Error Creating CUDA Stream\n");
                            printf("%s\n", cudaGetErrorString(error_stream));
                        }
                        cuda_current_device_num = device;
                        printf("Device %d registered\n", device);
                        threads_per_block = 256;
                        max_pd_block_size = threads_per_block * current_cudaDeviceProp.maxGridSize[0];
                        printf("CUDA Block size: %d\n",threads_per_block);
                        printf("Max Pd Block size: %d\n",max_pd_block_size);
                    }
                }
            }
            else
                printf("No CUDA devices detected\n");
        }
        else
            printf("Device number %d not recognized\n", device);
    }
    /*if (!cuda_stream)
    {
        printf("No CUDA device registered");
        return(1);
    }*/
    return(0);
}


int main(int arg, char *argv[])
{
  void* library = dlopen("/home/ajb/pdcuda/pdcuda-external/teste-2/cudalib.o", RTLD_LAZY);
float *d_A, *d_B;
int N = 64;
  t_float in[64];
  t_float out1[64];
  t_float out2[64];
  int      (*fptr)(int);

  if(library == NULL) {
    printf("error: %s\n", dlerror());
  } else {
    printf("success.\n");
    lib_set_cuda_device(0);


  cudaError_t errno;
  
  printf("initializing cuda~ object.\n");

  /* init pd object */
  N = 64;

  /* device memory */
  errno = cudaMalloc((void **) &d_A, N*sizeof(t_sample));
  if (errno)
    printf("cuda error (new): %d - %s\n", (int) errno, cudaGetErrorString(errno));

  errno = cudaMalloc((void **) &d_B, N*sizeof(t_sample));
  if (errno)
    printf("cuda error (new): %d - %s\n", (int) errno, cudaGetErrorString(errno));

    N=64;
    while (N--) {
      in[N]=(float)N;
    }

    

  errno = cudaMemcpy(d_A, in, (size_t) N*sizeof(t_sample), cudaMemcpyHostToDevice);
  if (errno)
    printf("cuda error (perform): %d - %s\n", (int) errno, cudaGetErrorString(errno));

  /*cuda_scalarplus_basic(d_A, d_B, 3.0, n);*/

  errno = cudaMemcpy(d_B, d_A, (size_t) N*sizeof(t_sample), cudaMemcpyDeviceToDevice);

  cudaSetDeviceFlags(cudaDeviceBlockingSync);

  /* find the address of function and data objects */
  *(void **)(&fptr) = dlsym(library, "cuda_zero_basic");

  /* invoke function, passing value of integer as a parameter */
  (*fptr)(d_B, N, threads_per_block, max_pd_block_size);

  cudaDeviceSynchronize();

  errno = cudaMemcpy(out2, d_B, (size_t) N*sizeof(t_sample), cudaMemcpyDeviceToHost);
  if (errno)
    printf("cuda error (perform): %d - %s\n", (int) errno, cudaGetErrorString(errno));



  }

  return 0;
}
