#include<cuda_runtime_api.h>
#include"m_pd.h"

extern "C" void get_grid_and_block(int *grid,int *block,int s_n);

__global__ void Zero(float* A)
{
    int i = blockDim.x * blockIdx.x + threadIdx.x;
    A[i] = 0;
}

__global__ void Sig(float* A, float f)
{
    int i = blockDim.x * blockIdx.x + threadIdx.x;
    A[i] = f;
}

__global__ void Plus(float* A, float* B, float *C)
{
    int i = blockDim.x * blockIdx.x + threadIdx.x;
    C[i] = A[i] + B[i];
}

__global__ void ScalarPlus(float* A, float* B, float f)
{
    int i = blockDim.x * blockIdx.x + threadIdx.x;
    B[i] = A[i] + f;
}

extern "C" void cuda_zero_basic(t_sample *vec, int n)
{
    int grid,block;
    get_grid_and_block(&grid, &block, n);
    Zero<<<grid,block>>>(vec);
}

extern "C" void cuda_sig_basic(t_sample *vec, t_float f, int n)
{
    int grid, block;
    get_grid_and_block(&grid, &block, n);
    Sig<<<grid,block>>>(vec, f);
}

extern "C" void cuda_scalarplus_basic(t_sample *in, t_sample *out, t_float f, int n)
{
    int grid, block;
    get_grid_and_block(&grid,&block,n);
    ScalarPlus<<<grid,block>>>(in, out, f);
}

extern "C" void cuda_plus_basic(t_sample *in1, t_sample *in2, t_sample *out, int n)
{
    int grid, block;
    get_grid_and_block(&grid,&block,n);
    Plus<<<grid,block>>>(in1, in2, out);
}

/*
    // epilog code for blocking: write buffer to parent patch 
static t_int *cuda_voutlet_doepilog(t_int *w)
{
    t_voutlet *x = (t_voutlet *)(w[1]);
    t_sample *out = (t_sample *)(w[2]);

    int n = (int)(w[3]);
    t_sample *in = x->x_empty;
    if (x->x_updown.downsample != x->x_updown.upsample)
        out = x->x_updown.s_vec;

#if 0
    if (tot < 5) post("outlet in %lx out %lx n %lx", in, out, n), tot++;
#endif
    for (; n--; in++) *out++ = *in, *in = 0;
    if (in == x->x_endbuf) in = x->x_buf;
    x->x_empty = in;
    return (w+4);
}
*/
